@@ -11,6 +11,7 @@ import urllib.request, urllib.parse, urllib.error
 import random
 from datetime import datetime
 import subprocess
+import platform
 import functools
 import requests
 import configobj
@@ -20,8 +21,7 @@ import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('AppStream', '1.0')
 gi.require_version('XApp', '1.0')
-gi.require_version('Flatpak', '1.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, GLib, Gio, Flatpak, XApp
+from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, GLib, Gio, XApp
 
 from installer import installer
 from misc import print_timing
@@ -426,8 +426,10 @@ class Application(Gtk.Application):
 
         self.gui_ready = False
 
+        self.low_res = self.get_low_res_screen()
+
         self.settings = Gio.Settings("com.linuxmint.install")
-        self.arch = Flatpak.get_default_arch()
+        self.arch = platform.machine()
 
         print("MintInstall: Detected system architecture: '%s'" % self.arch)
 
@@ -549,6 +551,14 @@ class Application(Gtk.Application):
                                            _("An error occurred attempting to add the Flatpak repo."))
                 res = dialog.run()
                 dialog.destroy()
+            elif error == "no-flatpak-support":
+                dialog = Gtk.MessageDialog(self.main_window,
+                                           Gtk.DialogFlags.MODAL,
+                                           Gtk.MessageType.ERROR,
+                                           Gtk.ButtonsType.OK,
+                                           _("Flatpak support is not currently available. Try installing flatpak."))
+                res = dialog.run()
+                dialog.destroy()
             elif error == "cancel":
                 pass
 
@@ -558,10 +568,51 @@ class Application(Gtk.Application):
         self.add_categories()
         self.installer.init(self.on_installer_ready)
 
-    def on_pkginfo_from_uri_complete(self, pkginfo):
+    def on_pkginfo_from_uri_complete(self, pkginfo, error=None):
+        if error:
+            if error == "no-flatpak-support":
+                dialog = Gtk.MessageDialog(self.main_window,
+                                           Gtk.DialogFlags.MODAL,
+                                           Gtk.MessageType.ERROR,
+                                           Gtk.ButtonsType.OK,
+                                           _("Flatpak support is not currently available. Try installing flatpak and gir1.2-flatpak-1.0."))
+                res = dialog.run()
+                dialog.destroy()
+                self.finish_loading_visual()
+                return
         if pkginfo:
             self.show_package(pkginfo, self.PAGE_LANDING)
 
+    def get_low_res_screen(self):
+        display = Gdk.Display.get_default()
+        n_monitors = display.get_n_monitors()
+
+        # Find the shortest monitor height.
+
+        min_height = 99999
+
+        for i in range(n_monitors):
+            monitor = display.get_monitor(i)
+
+            # Look only at geometry, not workarea - this we're looking at the monitor itself,
+            # not affected by someone's panel configuration.
+            rect = monitor.get_geometry()
+
+            min_height = rect.height if rect.height < min_height else min_height
+
+        # If it's less than our threshold than consider us 'low res'.  It's possible we
+        # have false positives here, especially with multi-monitor setups.  We'll always
+        # go off the lowest res monitor, but at the point this is called, we don't know
+        # which monitor we'll spawn on.  Knowing this would require waiting for window
+        # realize, and this really isn't that big a deal.  The workarea being used is in
+        # app pixels, so hidpi will also be affected here regardless of device resolution.
+
+        if min_height < 1000:
+            print("MintInstall: low resolution monitor detected, limiting window height.")
+            return True
+
+        return False
+
     def create_window(self, starting_page):
         if self.main_window != None:
             print("MintInstall: create_window called, but we already had one!")
@@ -589,6 +640,9 @@ class Application(Gtk.Application):
         self.active_tasks_button = self.builder.get_object("active_tasks_button")
         self.active_tasks_spinner = self.builder.get_object("active_tasks_spinner")
         self.no_packages_found_label = self.builder.get_object("no_packages_found_label")
+        
+        self.no_packages_found_refresh_button = self.builder.get_object("no_packages_found_refresh_button")
+        self.no_packages_found_refresh_button.connect("clicked", self.on_refresh_cache_clicked)
 
         self.progress_label = DottedProgressLabel()
         self.progress_box.pack_start(self.progress_label, False, False, 0)
@@ -630,6 +684,16 @@ class Application(Gtk.Application):
         separator.show()
         submenu.append(separator)
 
+        self.refresh_cache_menuitem = Gtk.MenuItem(_("Refresh the list of packages"))
+        self.refresh_cache_menuitem.connect("activate", self.on_refresh_cache_clicked)
+        self.refresh_cache_menuitem.show()
+        self.refresh_cache_menuitem.set_sensitive(False)
+        submenu.append(self.refresh_cache_menuitem)
+
+        separator = Gtk.SeparatorMenuItem()
+        separator.show()
+        submenu.append(separator)
+
         about_menuitem = Gtk.MenuItem(_("About"))
         about_menuitem.connect("activate", self.open_about)
         about_menuitem.show()
@@ -696,29 +760,56 @@ class Application(Gtk.Application):
         self.listbox_categories_selected_id = self.listbox_categories.connect('row-activated',
                                                                               self.on_row_activated)
 
+    def refresh_cache(self):
+        self.builder.get_object("loading_spinner").start()
+        self.refresh_cache_menuitem.set_sensitive(False)
+
+        self.page_stack.set_visible_child_name(self.PAGE_LOADING)
+
+        self.installer.force_new_cache(self._on_refresh_cache_complete)
+
+    def _on_refresh_cache_complete(self):
+        self.add_categories()
+        self.installer.init(self.on_installer_ready)
+
+    def on_refresh_cache_clicked(self, widget, data=None):
+        self.refresh_cache()
+
     def on_installer_ready(self):
-        self.process_matching_packages()
+        try:
+            self.process_matching_packages()
+            self.refresh_cache_menuitem.set_sensitive(True)
 
-        self.apply_aliases()
+            self.apply_aliases()
 
-        self.load_featured_on_landing()
-        self.load_picks_on_landing()
-        self.load_categories_on_landing()
+            self.load_featured_on_landing()
+            self.load_picks_on_landing()
+            self.load_categories_on_landing()
 
+            self.sync_installed_apps()
+            self.update_conditional_widgets()
+            
+            GObject.idle_add(self.finished_loading_packages)
 
-        self.sync_installed_apps()
-        self.update_conditional_widgets()
-        
-        GObject.idle_add(self.finished_loading_packages)
+            # Can take some time, don't block for it (these are categorizing packages based on apt info, not our listings)
+            GObject.idle_add(self.process_unmatched_packages)
 
-        # Can take some time, don't block for it (these are categorizing packages based on apt info, not our listings)
-        GObject.idle_add(self.process_unmatched_packages)
-
-        self.review_cache = reviews.ReviewCache()
-        housekeeping.run()
+            self.review_cache = reviews.ReviewCache()
+            housekeeping.run()
+        except:
+            GObject.idle_add(self.refresh_cache)
 
     def load_featured_on_landing(self):
         box = self.builder.get_object("box_featured")
+
+        if self.low_res:
+            box.hide()
+
+            # This overrides the glade 800x600 defaults. 300 is excessively small so the window works
+            # out its own minimum height.
+            self.main_window.set_default_geometry(800, 300)
+            return
+
         for child in box.get_children():
             child.destroy()
 
@@ -769,6 +860,9 @@ class Application(Gtk.Application):
         box.show_all()
 
     def load_picks_on_landing(self):
+        if self.low_res:
+            self.builder.get_object("editor_picks_label").hide()
+
         box = self.builder.get_object("box_picks")
         for child in box.get_children():
             child.destroy()
@@ -836,12 +930,14 @@ class Application(Gtk.Application):
         button.connect("clicked", self.category_button_clicked, self.picks_category)
         flowbox.insert(button, -1)
 
-        # Add flatpaks
-        button = Gtk.Button()
-        button.set_label(self.flatpak_category.name)
-        button.connect("clicked", self.category_button_clicked, self.flatpak_category)
+        if self.installer.list_flatpak_remotes():
+            # Add flatpaks
+            button = Gtk.Button()
+            button.set_label(self.flatpak_category.name)
+            button.connect("clicked", self.category_button_clicked, self.flatpak_category)
+
+            flowbox.insert(button, -1)
 
-        flowbox.insert(button, -1)
         box.pack_start(flowbox, True, True, 0)
         box.show_all()
 
@@ -1129,14 +1225,16 @@ class Application(Gtk.Application):
 
         self.installer = installer.Installer()
 
-        from installer.pkgCache import cache
+        from installer import cache
+
+        self.installer.cache = cache.PkgCache(self.installer.have_flatpak)
+        self.installer.force_new_cache()
+        self.installer.backend_table = {}
 
-        self.installer.cache = cache.PkgCache()
-        self.installer.cache.force_new_cache()
         self.installer.initialize_appstream()
+        self.installer.generate_uncached_pkginfos(self.installer.cache)
 
         self.add_categories()
-        self.build_matched_packages()
         self.process_matching_packages()
         self.process_unmatched_packages()
 
@@ -1181,6 +1279,10 @@ class Application(Gtk.Application):
             if res == Gtk.ResponseType.NO:
                 return True
 
+        # kill -9 won't kill mp subprocesses, we have to do them ourselves.
+        housekeeping.kill()
+        self.review_cache.kill()
+
         # Not happy with Python when it comes to closing threads, so here's a radical method to get what we want.
         os.system("kill -9 %s &" % os.getpid())
 
@@ -1539,7 +1641,7 @@ class Application(Gtk.Application):
         else:
             self.show_subcategories(category)
 
-        self.show_packages(category.pkginfos)
+        self.show_packages(category.pkginfos, from_search=False)
 
         self.update_conditional_widgets()
 
@@ -1689,7 +1791,7 @@ class Application(Gtk.Application):
 
     def on_search_results_complete(self, results):
         self.clear_category_list()
-        self.show_packages(results)
+        self.show_packages(results, from_search=True)
 
     def on_flowbox_item_clicked(self, tile, data=None):
         # This ties the GtkButton.clicked signal for the Tile class
@@ -1730,7 +1832,7 @@ class Application(Gtk.Application):
         else:
             return (string)
 
-    def show_packages(self, pkginfos):
+    def show_packages(self, pkginfos, from_search=False):
         if self.one_package_idle_timer > 0:
             GObject.source_remove(self.one_package_idle_timer)
             self.one_package_idle_timer = 0
@@ -1741,11 +1843,18 @@ class Application(Gtk.Application):
         self.category_tiles = []
         if len(pkginfos) == 0:
             self.app_list_stack.set_visible_child_name("no-results")
+
             if self.current_category == self.active_tasks_category:
                 text = _("All operations complete")
+                self.no_packages_found_refresh_button.hide()
             else:
-                text = _("No matching packages found")
-
+                if from_search:
+                    text = _("No matching packages found")
+                    self.no_packages_found_refresh_button.hide()
+                else:
+                    text = _("No packages to show.\nThis may indicate a problem - try refreshing the cache.")
+                    self.no_packages_found_refresh_button.show()
+                    
             self.no_packages_found_label.set_markup("<big><b>%s</b></big>" % text)
         else:
             self.app_list_stack.set_visible_child_name("results")
